<?php

require_once("site.config.php");

/* SITEATION ERROR CONSTANTS **************************************************/
// No error
define("seOkay", 0);

// GENERAL ERRORS
// One or more of the parameters supplied to a function was invalid
define("seInvalidParam", -400);

// DATA SOURCE ERRORS
// Generic data source error
define("seDataSource", -503);
// Unable to connect to the data source or open the specified database
define("seDataUnavailable", -503);

// QUERY ERRORS
// Generic query error
define("seQuery", -500);

// TEMPLATE ERRORS
// Generic template error
define("seTemplate", -404);
// The specified template could not be found
define("seTemplateUnavailable", -404);

// SITE CONFIGURATION ERRORS
// Generic site configuration error
define("seSiteConfig", -500);

// SESSION ERRORS
// Generic session error
define("seSession", -401);
// The specified session has expired
define("seSessionExpired", -401);

// RESOURCE (Channel, View, Post) ERRORS
// Generic error
define("seResource", -404);
// The specified channel/view/post was not found
define("seResourceNotFound", -404);
// The user does not have permission to operate on the specified resource
define("seResourceForbidden", -403);


/* SITEATION ERROR MESSAGES ***************************************************/
$SITE_ERROR_MESSAGES = array (
  seOkay => "The requested operation completed successfully.",
  -200 => "OK",
  -400 => "Bad Request - One or more of the parameters specified was invalid.",
  -401 => "Unauthorized - The current session has expired.",
  -403 => "Forbidden - Access to the specified resource was denied.",
  -404 => "Not Found - The resource requested is unavailable.",
  -500 => "Internal Server Error - A resource required by the site software is unavailable.",
  -503 => "Internal Server Error - A resource required by the site software is unavailable.",
);


// If we are in debug mode, this array will hold debugging information
$SITE_DEBUG = array();

// Adds a debug message to the debugging array
function DebugOut($caller, $message) {
  // Add the message (if we're debugging)
  if (SITE_DEBUG > 0) {
    global $SITE_DEBUG;
    // Combine the $message array into a single string
    if (is_array($message)) $message = implode("", $message);
    // Store the message and timestamp
    $SITE_DEBUG[] = "<div><strong>$caller</strong> reported at ".strftime("%c").":</div><div style='padding-left:10px'>$message</div>";
  }
}

// Displays the site debug console
function ShowDebugConsole() {
  // If debugging is not enabled, there's no need for the console
  if (SITE_DEBUG != 1) return;
  echo <<< DEBUG_SCRIPT_TOP
  <!-- BEGIN DEBUG CONSOLE ---------------------------------------------------->
  <script language="JavaScript">
    siteDebugWindow = window.open("", "Site_Debug_Console", "width=1100,height=600,scrollbars=yes");
    siteDebugWindow.document.writeln("<html><title>Site Debug Console</title>");
    siteDebugWindow.document.writeln("<body style='font:10pt Tahoma'>");
DEBUG_SCRIPT_TOP;
  global $SITE_DEBUG;
  // Output the execution timeline
  echo("siteDebugWindow.document.writeln(\"<table style='border:solid 2px #000080;font:10pt Tahoma;width:100%'>\");\n");
  echo("siteDebugWindow.document.writeln(\"<tr><th bgcolor='navy' colspan=4><font color='white'>Execution Timeline</font></th></tr>\");\n");
  foreach($SITE_DEBUG as $key => $value) {
    echo("siteDebugWindow.document.write(\"<tr><td style='padding:2pt;text-align:center;font:8pt Tahoma,Helvetica;background-color:gray;color:whitesmoke'><strong>$key</strong></td>\");\n");
    echo("siteDebugWindow.document.writeln(\"<td style='padding:2pt;font:8pt Tahoma, Helvetica;background:lightgray'>".$value."</td></tr>\");\n");
  }
  echo("siteDebugWindow.document.writeln(\"</table>\")\n");
  echo <<< DEBUG_SCRIPT_BOTTOM
  siteDebugWindow.document.writeln("</body></html>");
  siteDebugWindow.document.close();
  </script>
  <!-- END DEBUG CONSOLE ------------------------------------------------------>
DEBUG_SCRIPT_BOTTOM;
}

// Displays a pretty error message to the user before killing the script
function DiePrettily($errorCode, $params = "") {
  GLOBAL $SITE_ERROR_MESSAGES;
  // See if there are any additional params (convert them to a string)
  if (is_array($params)) {
    // We have an array of parameters.  Is it associative?
    if (is_assoc($params)) $params = implode_assoc($params, "; ", ":");
    else $params = implode("", $params);
  }
  if (!is_string($params)) $params = print_r($params, TRUE);
  // Encode the message and parameters into one string
  $message = $SITE_ERROR_MESSAGES[$errorCode];
  $errorCode = -($errorCode);
  // Format error output only if we are not using AJAX
  if ((isset($_GET["ajax"])) || (isset($_POST["ajax"]))) {
    header("HTTP/1.1 $errorCode $message");
    echo ("Error $errorCode: $message\nAdditional Information: $params");
  }
  else {
    GLOBAL $SITE_CONFIG;
    $webmasterEmail = $SITE_CONFIG["SITE_WEBMASTER"];
    DebugOut("DiePrettily()", array(
      "A routine requested program death because of an unrecoverable error.",
      "  The error code provided was <strong>$errorCode</strong>.  ",
      "The error message was:<div style='padding-left:10px'>&lt;", $message, "&gt;</div>",
      "Additional information:<div style='padding-left:10px'>", $params, "</div>"));
    echo("<html>\n  <head>\n  <meta http-equiv=\"content-type\" "); 
    echo("content=\"text/html; charset=windows-1250\">\n");
    echo("  <title>Error $errorCode</title>\n");
    // Show the debugging console
    ShowDebugConsole();
    echo("\n  </head>\n");
echo <<< ERROR_OUTPUT
  <body>
  <div style="border:solid 4px darkred;width:50%;position:relative;top:25%;left:25%">
    <span style="font:bold italic 48pt serif;background-color:darkred;color:white;float:right;width:50pt;height:50pt;text-align:center;border-left:solid 4px darkred;border-bottom:solid 4px darkred">
      &nbsp;!&nbsp;
    </span>  
    <div style="background-color:darkred;padding:2pt;font:bold 14pt sans-serif;color:white">
      D'oh!
    </div>
    <div style="font:10pt sans-serif;background-color:lightgray;padding:2pt">
      Unfortunately, the following error was encountered while attempting to
      accomodate your request:
    </div>
    <div style="font:bold 10pt sans-serif;background-color:lightgray;padding-left:10pt;padding-top:2pt;padding-bottom:2pt;padding-right:2pt">
        Error $errorCode:
    </div>
    <div style="font:10pt sans-serif;background-color:lightgray;padding-left:20pt;padding-bottom:2pt;padding-right:2pt">
      $message
    </div>
    <div style="font:bold 10pt sans-serif;background-color:lightgray;padding-left:10pt;padding-top:2pt;padding-bottom:2pt;padding-right:2pt">
        Additional Information:
    </div>  
    <div style="font:10pt sans-serif;background-color:lightgray;padding-left:20pt;padding-bottom:2pt;padding-right:2pt">
      $params
    </div>
    <div style="font:10pt sans-serif;background-color:lightgray;padding:2pt">
      Kindly <a href="mailto:$webmasterEmail?subject=Website+Error+$errorCode"
                onMouseOver="window.status='Inform the website administrator of this error.';return true"
                onMouseOut="window.status='';return true"
                title="Inform the website administrator of this error">inform 
      the website administrator</a> of this problem and the error
      message as it appears above so that the issue can be resolved as quickly
      as possible.  Thank you!
    </div>
  </div>
  </body>
</html>
ERROR_OUTPUT;
  }
  // Flush the output
  ob_end_flush();
  // Die
  die($errorCode);
}

?>
