// Returns a string representing today's date
function todaysDate() {
  var today = new Date();
  var month = today.getMonth() + 1;
  var date = today.getDate();
  var minutes = today.getMinutes();
  var seconds = today.getSeconds();
  // Create a string representing today's date
  var result = today.getFullYear() + "-";
  if (month < 10) result += "0" + month + "-";
  else result += month + "-";
  if (date < 10) result += "0" + date + " ";
  else result += date + " ";
  result += today.getHours() + ":";
  if (minutes < 10) result += "0" + minutes + ":";
  else result += minutes + ":";
  if (seconds < 10) result += "0" + seconds;
  else result += seconds;
  return result;
}

// Encodes form data into a string
function encodeFormData(docForm) {
  var submitContent = '';
  var formElem;
  var lastElemName = '';

  for (i = 0; i < docForm.elements.length; i++) {

    formElem = docForm.elements[i];
    switch (formElem.type) {
      // Text fields, hidden form elements
      case 'text':
      case 'hidden':
      case 'password':
      case 'textarea':
      case 'select-one':
        submitContent += formElem.name + '=' + escape(formElem.value) + '&'
        break;

      // Radio buttons
      case 'radio':
        if (formElem.checked) {
          submitContent += formElem.name + '=' + escape(formElem.value) + '&'
        }
        break;

      // Checkboxes
      case 'checkbox':
        if (formElem.checked) {
          // Continuing multiple, same-name checkboxes
          if (formElem.name == lastElemName) {
            // Strip of end ampersand if there is one
            if (submitContent.lastIndexOf('&') == submitContent.length-1) {
              submitContent = submitContent.substr(0, submitContent.length - 1);
            }
            // Append value as comma-delimited string
            submitContent += ',' + escape(formElem.value);
          }
          else {
            submitContent += formElem.name + '=' + escape(formElem.value);
          }
          submitContent += '&';
          lastElemName = formElem.name;
        }
        break;

    }
  }
  // Remove trailing separator
  submitContent = submitContent.substr(0, submitContent.length - 1);
  return submitContent;
}

/* Validates the specified form */
function validateForm(thisForm) {
  // If no required fields are specified, return true by default
  if ((!thisForm.required) || (thisForm.required.value == "")) {
    // Ensure the form is only submitted ONCE
    thisForm.onsubmit = NULL;
    return true;
  }
  // Retrieve the list of all required fields
  var requiredFields = thisForm.required.value.split(",");
  // Loop through the list of all required fields
  for (var i=0; i < requiredFields.length; i++) {
    // Determine the type of the required field
    fieldNameType = requiredFields[i].split("|");
    fieldName = fieldNameType[0];
    fieldType = fieldNameType[1];
    switch(fieldType) {
      // Verify the field is a valid e-mail address
      case "email":
        // Look for the @ symbol and at least one period following it
        emailDest = thisForm.elements[fieldName].value.indexOf("@");
        emailDomain = thisForm.elements[fieldName].value.lastIndexOf(".");
        if ((emailDest < 0) || (emailDomain < 0) || (emailDomain <= emailDest)) {
          alert("Please enter a valid e-mail address before continuing.");
          return false;
        }
        break;
      // Verify (and scramble) a password field
      case "password":
        // Ensure the field is not empty
        if (thisForm.elements[fieldName].value == "") {
          alert("Please enter a password before continuing.");
          return false;
        }
        // Scramble the password and store it in the field
        thisForm.elements["password"].value = hex_md5(thisForm.elements[fieldName].value);
        // Clear the password field
        thisForm.elements[fieldName].value = "";
        break;
      default:
        // Ensure the field is not empty, at least
        if (thisForm.elements[fieldName].value == "") {
          alert("Please enter " + fieldType + " before continuing.");
          return false;
        }
    }
  }
  return true;
}

// Retrieves the specified cookie
// From: http://techpatterns.com/downloads/javascript_cookies.php
function valueOfCookie(check_name) {
	// first we'll split this cookie up into name/value pairs
	// note: document.cookie only returns name=value, not the other components
	var a_all_cookies = document.cookie.split( ';' );
	var a_temp_cookie = '';
	var cookie_name = '';
	var cookie_value = '';
	var b_cookie_found = false; // set boolean t/f default f

	for ( i = 0; i < a_all_cookies.length; i++ )
	{
		// now we'll split apart each name=value pair
		a_temp_cookie = a_all_cookies[i].split( '=' );
		// and trim left/right whitespace while we're at it
		cookie_name = a_temp_cookie[0].replace(/^\s+|\s+$/g, '');
		// if the extracted name matches passed check_name
		if ( cookie_name == check_name )
		{
			b_cookie_found = true;
			// we need to handle case where cookie has no value but exists (no = sign, that is):
			if ( a_temp_cookie.length > 1 )
			{
				cookie_value = unescape(a_temp_cookie[1].replace(/^\s+|\s+$/g, ''));
			}
			// note that in cases where cookie is initialized but no value, null is returned
			return cookie_value;
			break;
		}
		a_temp_cookie = null;
		cookie_name = '';
	}
	if ( !b_cookie_found )
	{
		return null;
	}
}

// Toggles the visibility of the specified element 
function toggleHidden(whichArea) {
  element = document.getElementById(whichArea);
  if (!element) return;
  switch(element.tagName) {
    case "DIV":
      if (element.style.display != "block") element.style.display = "block";
      else element.style.display = "none";
      break;
    default:
      if (element.style.display != "inline") element.style.display = "inline";
      else element.style.display = "none";
  }
}

function browserType() {
  if (navigator.userAgent.indexOf("MSIE") >= 0) {
    if (navigator.userAgent.indexOf("Opera") >= 0) return("Opera");
    else return("MSIE");
  }
  else if (navigator.userAgent.indexOf("Firefox") >= 0) return("Firefox");
  else if (navigator.userAgent.indexOf("Safari") >= 0) return("Safari");
  else return("Unknown");
}

/* Fixes page presentation based on browser (different browsers handle CSS 
   differently) */
function fixPresentation() {
  mainframe = document.getElementById("MainFrame");
  sidebar = document.getElementById("Sidebar");
  if (browserType() == "MSIE") {
      mainframe.style.width = document.body.offsetWidth - 179;
      mainframe.style.height = document.body.offsetHeight;
  } 
  else { 
    mainframe.style.width = document.body.scrollWidth - 200;
    mainframe.style.height = document.body.scrollHeight;
  }
}
