<?php
/* postit.php
   Defines the Siteation poster, which allows posts to be added and edited.
   Part of Siteation.  Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.config.php");
require_once("libs/site.error.php");
require_once("libs/site.lib.php");
require_once("libs/session.lib.php");
require_once("libs/user.lib.php");

/* The login script accepts the following variables via HTTP POST.  Note that
   none of these variables are case-sensitive and that the order in which they
   are passed makes no difference.

   UserName       - Specifies the username of the user attempting to log in.

   Password       - Specifies an MD5-hashed value that represents the password
                    entered by the user.
                    
   Session        - Specifies the current session ID of the user.  This session
                    will be invalidated and a new one will be assigned if the
                    user successfully logs in.
                    
   Success        - Specifies the name of the view to which the client will be
                    redirected if the post action succeeds.  If this parameter
                    is not specified, or it is blank, the client will be
                    redirected to the previous view.
                    
   These parameters must be the names given to Form Submit items in your
   HTML document.  However, case is unimportant; "DeSCRIption" is the
   same as "description" as far as the poster is concerned.
   
   This script also accepts two HTTP GET parameters:
   
   Logout         - Specifies that the current user's session is to be
                    invalidated, thus logging them out

   Success        - Specifies the channel that the user will be redirected to
                    after they are logged out
*/

/* MAIN ***********************************************************************/

// Buffer output
ob_start();

/* This code prevents client browsers and intermediate proxies from caching
   the output of this script.
*/
// Set the expiration date to a date in the past
header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
// Indicate that the content is always modified
header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
// HTTP/1.1 headers that will prevent buffering
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Cache-Control: post-check=0, pre-check=0", false);
// HTTP/1.0 headers this will prevent buffering
header("Pragma: no-cache");

// If no parameters were passed, die a horrible death
if ((count($_GET) == 0) && (count($_POST) == 0))
  DiePrettily(seResourceForbidden, array(
    "Login",
    " - One or more of the login credentials supplied was invalid. [5]"));

// Connect to the site database
$thisSite = new SITE();

// Start (or continue) a session
$thisSession = new SITE_SESSION($thisSite);

// This array will hold information sent by the client
$clientLogin = array();

// Holds a message for the user
$message = "";

// Retrieve the parameters passed
$clientLogin = array_merge($_GET, $_POST);
// Change all of the key names to lowercase
$clientLogin = array_change_key_case($clientLogin);
    
// If the user wants to log out...
if (isset($clientLogin["logout"])) {
    // Invalidate the current session
    session_destroy();
    // Create a new session for the anonymous site visitor
    //$thisSession->Create($thisSite->Setting("Defaults", "VisitorAccount"));
    // Set the message that will be displayed to the user
    if (isset($clientLogin["ajax"])) 
      $message = $thisSite->Setting("Defaults", "VisitorAccount")." is now logged in.";
    else $message = "You are now logged out.";
}
// If parameters were passed via POST...
else {
  // Set this in case we encounter an error
  $_SESSION["LastAction"] = "ERROR";
  // If no username was specified, indicate an error
  if (!isset($clientLogin["username"]) || empty($clientLogin["username"]))
    DiePrettily(seResourceForbidden, array(
      "Login",
      " - One or more of the login credentials supplied was invalid."));
  //$clientLogin["session"] = urldecode($clientLogin["session"]);
  //$correctSession = bin2hex(md5($thisSession->Setting("SessionID"), true));
  // If no session (or the wrong session) was specified, indicate an error
  //if (!isset($clientLogin["session"]) || empty($clientLogin["session"])
  //  || $clientLogin["session"] != $correctSession)
  //  DiePrettily(seResourceForbidden, array(
  //    "Login",
  //    " - One or more of the login credentials supplied was invalid. [2]",
  //    "The session ID received was: ", $clientLogin["session"],
  //    " but it should have been: ", $correctSession));
  /* Retrieve information about the specified user */
  $thisUser = new SITE_USER($thisSite);
  // Get the user's information
  $result = $thisUser->Get($clientLogin["username"]);
  // If the user was not found, indicate an error
  if ($result < 0)
    DiePrettily(seResourceForbidden, array(
      "Login",
      " - One or more of the login credentials supplied was invalid."));
  // If the account is disabled, indicate an error
  if ($thisUser->Setting("Enabled") == "N")
    DiePrettily(seResourceForbidden, array(
      "Login",
        " - The account requested -- ", $clientLogin["username"],
        " -- is disabled."));
  // Update the visit timestamp for this user
  $thisUser->Set("LastVisit", $thisSite->DataSource()->TimeValue());
  // Check the session ID
  if ((!isset($clientLogin["session"])) || ($clientLogin["session"] != session_id()))
    DiePrettily(seResourceForbidden, array(
      "Login",
      " - One or more of the login credentials supplied was invalid.",
      "Received: ", $clientLogin["session"]));
  $sessionPwd = $thisUser->Setting("UserPassword");
  // If no password (or the wrong password) was specified, indicate an error
  if ((!isset($clientLogin["password"])) || ($clientLogin["password"] != $sessionPwd)) {
     // Retrieve the number of failed login attempts by this account
    $failedLogins = $thisUser->Setting("FailedLogins") + 1;
    // Update the number of failed login attempts for this account
    $thisUser->Set("FailedLogins", $failedLogins);
    // Retrieve the lockout threshold
    $lockoutThreshold = $thisSite->Setting("Security","LockoutThreshold");
    /* If this account is NOT 'Administrator' and we've exceeded the lockout
       threshold, disable the account */
    if (($thisUser->Setting("UserName") != "Administrator")
      && ($failedLogins >= $lockoutThreshold)) 
      $thisUser->Set("Enabled","N");
    DiePrettily(seResourceForbidden, array(
      "Login",
      " - One or more of the login credentials supplied was invalid."));
  }
  // If we make it here, then the credentials specified were valid!
  // Invalidate the current session
  //$thisSession->Invalidate();
  // Refresh the current session
  $_SESSION["UserName"] = $thisUser->Setting("UserName");
  if ((isset($clientLogin["persist"])) && 
    ($this->site->Setting("Sessions", "AllowPersistent") == "Y"))
      $_SESSION["Persist"] = "Y";
  $thisSession->Renew();
  // Create a new session specifically for the user
  //if ($clientLogin["persist"] == "on")
    // Create a persistent login
  //  $thisSession->Create($thisUser->Setting("UserName"), TRUE);
  //else $thisSession->Create($thisUser->Setting("UserName"));
  // Reset the failed login counter
  $thisUser->Set("FailedLogins", "0");
  // Set the last action
  $_SESSION["LastAction"] = "LOGIN";
  // Set the message to display to the user
  $message = $thisUser->Setting("UserName"). " is now logged in.";
  /* Clean up */
  $thisUser->__destruct();
}
// Redirect the client to the last channel they were viewing
$redirect = "2;URL=http://".$_SERVER["HTTP_HOST"];
$redirect .= $SITE_CONFIG["SCRIPT_URL"]."/viewpoint.php?channel=";
$redirect .= urlencode($_SESSION["LastChannel"]);

// Flush all session information until the session is resumed (or it expires)
$thisSession->__destruct();
// Flush all site information
$thisSite->__destruct();

/* Output a pseudo-HTML page so that the cookie can be cleared and reset */
/* See if we can get the special template for site messages */
if (isset($clientLogin["ajax"])) echo($message);
else {
  echo <<< UGLY_MESSAGE
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
  <head>
  <meta http-equiv="content-type" content="text/html; charset=windows-1250">
  <meta http-equiv="refresh" content="$redirect">
  <link rel="stylesheet" type="text/css" href="templates/default/default.css">
  <title>Post</title>
  </head>
  <body>
    <div class="Post">
      <div>
        <span class="PostTitle">Login</span>
      </div>
      <div class="PostBody">
        $message
      </div>
      <div class="PostFooter">&nbsp;</div>
    </div> 
</html>
UGLY_MESSAGE;
  // If we're debugging, show the debugging console
  if(SITE_DEBUG) ShowDebugConsole();
}
header("HTTP/1.1 200 OK");
// Flush the output buffer
ob_end_flush();
?>
