<?php

/* postit.php
   Defines the Siteation poster, which allows posts to be added and edited.
   Part of Siteation.  Copyright (C) 2005 Studio 703.  All rights reserved.
*/

require_once("site.config.php");
require_once("libs/site.lib.php");
require_once("libs/session.lib.php");
require_once("libs/channel.lib.php");
require_once("libs/user.lib.php");

/* The poster accepts the following variables via HTTP POST.  Note that none
   of these variables are case-sensitive and that the order in which they are
   passed makes no difference.

   action         - Specifies whether the post is NEW, a REPLY, an edit, or
                    if the post should be DELETEd.
   
   ID             - This parameter is ignored unless the post is either a REPLY
                    or the user has permission to edit and delete posts.  If
                    this post is a REPLY, this parameter contains the ID of
                    the parent post.
                    
   Channel        - This parameter specifies the channel to which the post
                    belongs.
                    
   Title          - Specifies the title of the post.
   
   Description    - Specifies a description of the post.  If this parameter
                    is not supplied, or if it is empty, the poster will
                    generate a description using the first 255 characters of
                    the post data.
                    
   Icon           - Specifies the URL of an icon that can be attached to the
                    post.  The poster checks to ensure that the URL specified
                    ends in *.gif, *.jpg, *.jpeg, *.png; if it does not, this
                    field is discarded.
                    
   Keywords       - Defines one or more keywords that can be used when
                    searching for this post.  This field can be left blank.
                    
   Data           - Defines the acutal text of the post.  This field is
                    sanitized based on the tags allowed by the channel to
                    which it is being posted.
                    
   Success        - Specifies the name of the view to which the client will be
                    redirected if the post action succeeds.  If this parameter
                    is not specified, or it is blank, the client will be
                    redirected to the previous view.
                    
   These parameters must be the names given to Form Submit items in your
   HTML document.  However, case is unimportant; "DeSCRIption" is the
   same as "description" as far as the poster is concerned.
*/

/* MAIN ***********************************************************************/

// Buffer output
ob_start();

/* This code prevents client browsers and intermediate proxies from caching
   the output of this script.
*/
// Set the expiration date to a date in the past
header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
// Indicate that the content is always modified
header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
// HTTP/1.1 headers that will prevent buffering
header("Cache-Control: no-store, no-cache, must-revalidate");
header("Cache-Control: post-check=0, pre-check=0", false);
// HTTP/1.0 headers this will prevent buffering
header("Pragma: no-cache");

// Connect to the site database
$thisSite = new SITE();

// Start (or continue) a session
$thisSession = new SITE_SESSION($thisSite);

// Set this in case we encounter an error
$_SESSION["LastAction"] = "ERROR";

// This array is used to test that the action specified by the client is valid
$validActions = array (
  "NEW" => "w",
  "REPLY" => "y",
  "EDIT" => "m",
  "DELETE" => "d");

// This array will hold information sent by the client
$clientPost = array();

// This array will hold information sent by the client
$clientLogin = array();

// Holds a message for the user
$message = "";

// Retrieve the parameters passed
$clientPost = array_merge($_GET, $_POST);
// Change all of the key names to lowercase
$clientPost = array_change_key_case($clientPost);

// If no channel was specified, indicate an error
if (!isset($clientPost["channel"]))
  DiePretty(seInvalidParam, array(
    "POSTIT",
    " - No channel was specified; therefore, the posting system does not know ",
    "where to place the specified post."));

// If no action was specified, indicate an error
if (!isset($clientPost["action"]))
  DiePretty(seInvalidParam, array(
    "POSTIT",
    " - No action was specified; therefore, the posting system does not know ",
    " what to do with the specified post."));
    
// Change the action to uppercase to avoid case conflicts
$clientPost["action"] = strtoupper($clientPost["action"]);
    
// If an invalid action was specified, indicate an error
if (!isset($validActions[$clientPost["action"]]))
  DiePrettily(seInvalidParam, array(
    "POSTIT",
    " - An unknown action was specified.  The posting system does not know ",
    "how to perform &lt;", $clientPost["action"], "&gt; on the specified ",
    "post."));
    
// Retrieve information about the specified channel
$thisChannel = new SITE_CHANNEL($thisSite);
$result = $thisChannel->Get($clientPost["channel"]);
// If the channel doesn't exist, die with a pretty error message
if ($result < 0) DiePrettily(seResourceNotFound, array(
  "POSTIT",
  " - An unknown channel was specified.  The posting system was unable to ",
  "find the channel: ", $clientPost["channel"], "."));

/* Retrieve information about the current user */
$thisUser = new SITE_USER($thisSite);
// Get the user's information
$result = $thisUser->Get($_SESSION["UserName"]);
// If the user was not found, indicate an error
if ($result < 0) DiePrettily(seResource, array(
  "POSTIT",
  " - The posting system was unable to determine whether or not the user -- ",
  $thisSession->Setting("UserName"), " -- ",
  " has permission to modify posts for the channel: ",
  $clientPost["channel"], "."));

/* Find out if the user can access this channel */
$subscription = $thisUser->Subscription($thisChannel);

/* If we don't have an array containing information about the group
   subscription to this channel, there's a problem (the user probably cannot
   access the channel)
*/
if (!is_array($subscription)) DiePrettily($subscription, array( 
  "POSTIT",
  " - The posting system was unable to determine whether or not the user -- ",
  $thisSession->Setting("UserName")," -- ",
  " has permission to modify posts for the channel: ",
  $clientPost["channel"], "."));

// If the access rights have not been specified, indicate an error
if (empty($subscription["Rights"]))
  DiePrettily(seResourceNotFound, array( 
  "POSTIT",
  " - The posting system was unable to determine whether or not the user -- ",
  " -- ", $thisSession->Setting("UserName"),"&gt;",
  " has permission to modify posts for the channel: ",
  $clientPost["channel"], "."));

/* Match the access rights of the user against the action they wish to perform.
   If they do not have permission to perform the requested action, return an
   error
*/
if (strpos($subscription["Rights"], $validActions[$clientPost["action"]]) === FALSE)
  DiePrettily(seResourceForbidden,
    "POSTIT",
    " - The user -- ", $thisSession->Setting("UserName"), " -- ",
    "needs appropriate permissions in order to modify posts in the channel:",
    $clientPost["channel"], ".");

/* Set the last subscription used by the user in the session variables so that
   others can use it
*/
/* Set the last subscription used by the user in the session variables so that
   others can use it
*/
$_SESSION["Subscription"] = implode_assoc($subscription);
$_SESSION["UserRights"] = $subscription["Rights"];

// Create a query handler to manage the post
$thisPost = new Query($thisSite->DataSource());

// Set the query table
$thisPost->location = array($SITE_CONFIG["TABLE_PREFIX"]."posts");

if ($clientPost["action"] == "DELETE") {
  // Delete an existing post
    $thisPost->criteria = array(
      "ID", " = ", Quote($clientPost["id"]));
    // Run the query
    $result = $thisPost->Delete();
    $message = "Your post has been deleted.";
}
else {
// If no post data was specified, indicate an error
if (!isset($clientPost["data"]) || empty($clientPost["data"]))
  DiePrettily(seInvalidParam, array(
    "POSTIT",
    " - No post content was provided; therefore, there is nothing to post."));

// Clean up the post (strip tags, add HTML entities, etc.)
// Clean up the title
$clientPost["title"] = Scrub($clientPost["title"]);
// Generate a list of keywords, if necessary
if (empty($clientPost["keywords"])) 
  $clientPost["keywords"] = implode(", ", GetKeywords($clientPost["data"]));
// Clean up the keywords  
$clientPost["keywords"] = Scrub($clientPost["keywords"]);
// Generate a description, if required
if (empty($clientPost["description"]))
  $clientPost["description"] = Summarize($clientPost["data"]);
// Clean up the description
$clientPost["description"] = Scrub($clientPost["description"]);
// Clean up the post data
$allowedTags = $thisChannel->Setting("AllowedTags");
if (empty($allowedTags)) $allowedTags = $thisSite->Setting("Channels", "DefaultTagsAllowed");
$clientPost["data"] = Scrub($clientPost["data"], $allowedTags, TRUE);
// Ensure that the icon URL specified points to a valid image
$allowedExtensions = array(
  "jpg" => TRUE,
  "jpeg" => TRUE,
  "gif" => TRUE,
  "png" => TRUE);
if (!empty($clientPost["icon"])) {
  // Get the extension
  $iconExtension = substr(strrchr(strtolower($clientPost["icon"]), "."), 1);
  // If the extension is invalid, dispose of the URL specified
  if (!isset($allowedExtensions[$iconExtension])) $clientPost["icon"] = "";
}

// Attempt to perform the action specified
switch ($clientPost["action"]) {
  // Create a new post
  case "NEW":
  case "REPLY":
    // Build the record to be inserted into the database
    $thisPost->fieldsValues = array(
    "ChannelName" => $clientPost["channel"],
    "Title" => $clientPost["title"],
    "Description" => $clientPost["description"],
    "Author" => $thisUser->Setting("UserName"),
    "Icon" => $clientPost["icon"],
    "Keywords" => $clientPost["keywords"],
    "CreateDate" => $thisSite->DataSource()->TimeValue(),
    "PubDate" => $thisSite->DataSource()->TimeValue(),
    "Content" => $clientPost["data"],
    "Extra" => "");
    // Track the user's IP address, if specified by the site settings
    if ($thisSite->Setting("Security", "IPTracking") != "N")
      $thisPost->fieldsValues["IP"] = $_SERVER["REMOTE_ADDR"];
    // Store extra post fields, if any are given
    if (isset($clientPost["extra"]) && !empty($clientPost["extra"])) {
      // Break out the extra field names
      $clientPost["extra"] = explode(", ", $clientPost["extra"]);
      $extraFields = array();
      foreach($clientPost["extra"] as $fieldName)
        if (isset($clientPost[$fieldName]))
          $extraFields[$fieldName] = $clientPost[$fieldName];
      $thisPost->fieldsValues["Extra"] = implode_assoc($extraFields); 
    }
    // We only need to add these if we're replying to an existing post
    if ($clientPost["action"] == "REPLY") {
      $thisPost->fieldsValues["Parent"] = $clientPost["parent"];
      $thisPost->fieldsValues["ThreadParent"] = $clientPost["ref"];
    }
    $result = $thisPost->Insert();
    $message = "Your post has been submitted.";
    break;
  // Edit an existing post
  case "EDIT":
    $thisPost->criteria = array(
      "ID", " = ", $clientPost["id"]);
    $thisPost->fieldsValues = array(
      "ChannelName" => $clientPost["channel"],
      "Title" => $clientPost["title"],
      "Description" => $clientPost["description"],
      "Author" => $thisUser->Setting("UserName"),
      "Icon" => $clientPost["icon"],
      "Keywords" => $clientPost["keywords"],
      "PubDate" => $thisSite->DataSource()->TimeValue(),
      "Content" => $clientPost["data"]);
    // Track the user's IP address, if specified by the site settings
    if ($thisSite->Setting("Security", "IPTracking") != "N")
      $thisPost->fieldsValues["IP"] = $_SERVER["REMOTE_ADDR"];
    // Run the query
    $result = $thisPost->Update();
    $message = "Your post has been updated.";
}
}
// If we were unsuccessful, report the error
if ($result <= 0) DiePrettily(seQuery, array(
  "POSTIT",
  " - An error occurred when the posting system attempted to perform the ",
  "specified action: ", $clientPost["action"], ". ",
  "The result was: ", $thisPost->Message(), "."));

// Update the channel's publication timestamp
$thisChannel->Set("PubDate", $thisSite->DataSource()->TimeValue());

// Update the user's post count
if ($clientPost["action"] != "DELETE") {
  $postCount = $thisUser->Setting("PostCount") + 1;
  $thisUser->Set("PostCount", $postCount);
}

// Set the last action
$_SESSION["LastAction"] = $clientPost["action"];
// Set the last post modified
$_SESSION["LastPost"] = $clientPost["id"];

/* Clean up */
$thisUser->__destruct();
$thisChannel->__destruct();

// Redirect the client to the specified view
$redirect = "2;URL=http://".$_SERVER["HTTP_HOST"];
$redirect .= $SITE_CONFIG["SCRIPT_PATH"]."viewpoint.php?channel=";
$redirect .= $clientPost["channel"];

// Flush all session information until the session is resumed (or it expires)
$thisSession->__destruct();
// Flush all site information
$thisSite->__destruct();

/* Output a pseudo-HTML page so that the cookie can be cleared and reset */
/* See if we can get the special template for site messages */
if (isset($clientPost["ajax"])) echo($message);
else {
  echo <<< UGLY_MESSAGE
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
  <head>
  <meta http-equiv="content-type" content="text/html; charset=windows-1250">
  <meta http-equiv="refresh" content="$redirect">
  <link rel="stylesheet" type="text/css" href="templates/default.css">
  <title>Post</title>
  </head>
  <body>
    <div class="Post">
      <div>
        <span class="PostHeader">Post It</span>
      </div>
      <div class="PostBody">
        $message
      </div>
      <div class="PostFooter">&nbsp;</div>
    </div> 
</html>
UGLY_MESSAGE;
  // If we're debugging, show the debugging console
  if(SITE_DEBUG) ShowDebugConsole();
}
header("HTTP/1.1 200 OK");
  // Flush the output buffer
ob_end_flush();
?>
